'===============================================================================
' WinLib.bi
' Functions library for Windows GUI
' Created from WinGUI.bi on September 08, 2022
' Last changes on October 13, 2023
'===============================================================================

#Include Once "windows.bi"
#Include "win/commctrl.bi"
#Include Once "win\commdlg.bi"
InitCommonControls


'-------------------------------------------------------------------------------
'  Window Functions
'-------------------------------------------------------------------------------

'The following three procedures shall be used to create and run a window. 
'The window callback function (WndProc) has to be written in the main program.
'Several windows may use the same window class and the same callback function,
'example: dodicat's custom messagebox, https://www.freebasic.net/forum/viewtopic.php?t=27725


Sub RegisterWindowClass(ByVal WinClassName As String, _ 
	ByVal WndProc As Function(ByVal hWnd As HWND, ByVal message As UINT, ByVal wParam As WPARAM, ByVal lParam As LPARAM) As LRESULT)
	'Register the window class with predefined parameters. Parameters of the Sub:
	'- WndProc = callback function 
	'- WinClassName = window class name
	
	Dim wcls As WNDCLASS     
		 
	With wcls
		.style         = CS_HREDRAW or CS_VREDRAW
		.lpfnWndProc   = WndProc
		.cbClsExtra    = 0
		.cbWndExtra    = 0
		.hInstance     = GetModuleHandle(0)
		.hIcon         = LoadIcon(0, IDI_APPLICATION)
		.hCursor       = LoadCursor(0, IDC_ARROW)
		.hbrBackground = Cast(HBRUSH, COLOR_WINDOW)
		.lpszMenuName  = 0
		.lpszClassName = StrPtr(WinClassName)
	End With
				
	If(RegisterClass(@wcls) = FALSE) Then
		 MessageBox(0, "Failed to register window class", "Error", MB_ICONERROR)
		 Exit Sub
	End If
	
End Sub


Function Window_New(ByVal WinClassName As String, _
	ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, ByVal Title As String, _
	ByVal Style As ULong = WS_OVERLAPPEDWINDOW Or WS_VISIBLE) As HWND
	'Create a new window. Parameter:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Title = Title
	'- Style = See below
	'- WinClassName = window class name
	
	'Style:
	'WS_OVERLAPPEDWINDOW Or WS_VISIBLE creates an initial visible overlapped window 
	'with the WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, 
	'and WS_MAXIMIZEBOX styles. The size of the window can be changed by by click-dragging
	'from one of the window corners. 
	'Other styles may be used, e.g. WS_OVERLAPPED Or WS_SYSMENU Or WS_VISIBLE in order
	'to create a non-resizable window.  

	Return CreateWindowEx(0, StrPtr(WinClassName), Title, Style, x, y, w, h, 0, 0, GetModuleHandle(0), 0)
	
End Function


Sub RunMessageLoop()
	'Run the message loop. The following parameters from the MSG structure will be 
	'returned to the windows callback function:
	'- hWnd = handle of the window
	'- message = message number
	'- wParam = additional message information
	'- lParam = additional message information
	
	Dim As MSG wMsg

	While(GetMessage(@wMsg, 0, 0, 0) <> FALSE)    
		TranslateMessage(@wMsg)
		DispatchMessage(@wMsg)
	Wend
	
End Sub	

      
Sub Window_GetSize(ByVal hwnd As HWND, ByRef x_left As Long, ByRef y_top As Long, _
	ByRef x_right As Long, ByRef y_bottom As Long)
	'Retrieves the coordinates of a window's client area. Parameters:
	'- hwnd = Handle of the window
	'- x-left = x-coordinate of the upper-left corner
	'- y_top = y-coordinate of the upper-left corner
	'- x_right = x-coordinate of the lower-right corner
	'- y_bottom = y-coordinate of the lower-right corner
	
	Dim As RECT rect
	
	'Store window size in rect:
	GetClientRect(hwnd, @rect)
	x_left = rect.left
	y_top = rect.top 
	x_right = rect.right
	y_bottom = rect.bottom
	
End Sub


'-------------------------------------------------------------------------------
' Controls (General Functions)
'-------------------------------------------------------------------------------

Function Control_Createfont(ByVal Font As String, ByVal h As Long = 20, ByVal w As Long = 10, _
	ByVal wt As Long = FW_NORMAL, ByVal it As Long = False, ByVal ul As Long = False, ByVal so As Long = False) As HFONT
   
	'Create a font. Parameters:
	'-Font = e.g. "Courier New", "Arial", "New Times Roman"
	'-h = Logical height of the font
	'-w = Logical average width of font
	'-wt = Font weight (e.g. FW_THIN, FW_NORMAL, FW_BOLD)
	'-it = Italic: True = yes
	'-ul = Underline: True = yes
	'-so = Strikout: True = yes
   
  Return CreateFont(h, w, 0, 0, wt, it, ul, so, ANSI_CHARSET, FALSE, FALSE, _
                     DEFAULT_QUALITY, DEFAULT_PITCH or FF_ROMAN, Font)
End Function


Sub Control_Setfont(ByVal hWndControl As HWND, ByVal hFont As HFONT)

	'Set a font for a control. Parameters:
	'- hWndControl = handle of the control
	'- hFont = handle of the font
	
  SendMessage(hWndControl, WM_SETFONT, Cast(WPARAM, hFont), True)
    
End Sub


Sub Control_Deletefont(ByVal hFont As HFONT)

	'Delete a font (handle hFont is not longer valid)
	
	DeleteObject(hFont)
	
End Sub


Sub Control_Resize(ByVal hWndControl As HWND, ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long)
	'Resize a control. Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	
	MoveWindow(hWndControl, x, y, w, h, True)
	
End Sub	


'-------------------------------------------------------------------------------
' Button
'-------------------------------------------------------------------------------

Function Button_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Label As String, ByVal Style As Ulong = 0, ByVal hwnd As HWND) As HWND
	'Create a new button. Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Label = label
	'- Style = style (e.g. WS_TABSTOP may be used for controls if desired)
	'- hwnd = handle of the window		
	
	Return CreateWindowEx(0, "BUTTON", Label, Style Or WS_VISIBLE Or WS_CHILD, x, y, w, h, hwnd, 0, 0, 0 )
	
End Function


'-------------------------------------------------------------------------------
' Checkbox
'-------------------------------------------------------------------------------

Function CheckBox_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Label As String, ByVal Style As Ulong = 0, ByVal hwnd As HWND) As HWND
	'Create a new checkbox (autocheckbox, the state switches automatically by clicking). 
	'Parameters:
	'- x, y = top left corner
	'- w = width 
	'- h = height
	'- label = label
	'- Style = style (e.g. WS_TABSTOP may be used for controls if desired)	
	'- hwnd = handle of the window		
	
	Return CreateWindowEx(0, "BUTTON", Label, Style Or WS_VISIBLE Or WS_CHILD Or BS_AUTOCHECKBOX, x, y, w, h,	hWnd, 0, 0, 0 )
												
End Function


Sub CheckBox_SetCheck(ByVal hWndCheck As HWND, ByVal CheckState As Long)
	'Set check status of the checkbox. Parameters:
	'- hWndCheck = Handle of the checkbox
	'- CheckState: 1 = checked, 0 = unchecked
    
  Select Case CheckState
		Case 1 
			SendMessage(hWndCheck, BM_SETCHECK, BST_CHECKED, 0) 
		Case 0
			SendMessage(hWndCheck, BM_SETCHECK, BST_UNCHECKED, 0)
		End Select
       
End Sub


Function CheckBox_GetCheck(ByVal hWndCheck As HWND ) As Long
	'Request the status of the checkbox. 0 = unchecked, 1 = checked. Parameters:
	'- hWndCheck = handle of the checkbox
	
	If SendMessage(hWndCheck, BM_GETCHECK, 0, 0) = BST_CHECKED Then Return 1 _
		Else Return 0
       
End Function


'-------------------------------------------------------------------------------
' Radiobutton
'-------------------------------------------------------------------------------

Function RadioButton_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Label As String, ByVal Style As ULong = 0, ByVal hwnd As HWND) As HWND
	'New radio button (if a radio button is clicked, its state will be
	'automatically set to "checked" and the states of the others in the group
	'will be set to "unchecked". Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- label = label
	'- Style: WS_GROUP must be used for the first radio button of a group
	'- hwnd = handle of the window
		
	Return CreateWindowEx(0, "BUTTON", Label, Style Or WS_VISIBLE Or WS_CHILD Or BS_AUTORADIOBUTTON, x, y, w, h,	hWnd, 0, 0, 0 )
												
End Function


Sub RadioButton_SetCheck(ByVal hWndRadio As HWND, ByVal CheckState As Long)
	'Set the state of a radio button. Parameters:
	'- hWndRadio = handle of the radio buttons
	'- CheckState: 1 = checked, 0 = unchecked  
    
  Select Case CheckState
		Case 1
			SendMessage(hWndRadio, BM_SETCHECK, BST_CHECKED, 0)
		Case 0
			SendMessage(hWndRadio, BM_SETCHECK, BST_UNCHECKED, 0)
		End Select
       
End Sub


Function RadioButton_GetCheck(ByVal hWndRadio As HWND ) As Long
	'Request the state of the radio buttons. 0 = unchecked, 1 = checked. Parameters:
	'- hWndRadio = handle of the radio buttons
	
	If SendMessage(hWndRadio, BM_GETCHECK, 0, 0) = BST_CHECKED Then Return 1 _
		Else Return 0
       
End Function


'-------------------------------------------------------------------------------
' Group Box
'-------------------------------------------------------------------------------

Function GroupBox_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, _
	ByVal h As Long, ByVal Label As String, ByVal hwnd As HWND) As HWND
	'Create a new group box. Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Label = label
	'- hwnd = handle of the window
	
	Return CreateWindowEx(0, "BUTTON", Label, WS_VISIBLE Or WS_CHILD Or BS_GROUPBOX, _
		x, y, w, h, hwnd, 0, 0, 0 )
	
End Function
										

'-------------------------------------------------------------------------------
' Label (Static Text)
'-------------------------------------------------------------------------------

Function Label_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal text As String, ByVal Style As Ulong = 0, ByVal hwnd As HWND) As HWND
	'Create a new Label (= static text). Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Style = WS_BORDER, SS_LEFT, SS_CENTER, SS_RIGHT (different styles can be connected by "Or")
	'- text = Text
'- hwnd = handle of the window	
	
	Return CreateWindowEx(0, "STATIC", Text, Style Or WS_VISIBLE Or WS_CHILD, x, y, w, h, hWnd, 0, 0, 0 )
												
End Function


Sub Label_SetText(ByVal hWndLabel As HWND, ByVal Text As String)
	'Set text of a label. Parameters:
	'- hWndLabel = Handle of the label
	'- Text = text
    
  SetWindowText(hWndLabel, Text)
	
End Sub


'-------------------------------------------------------------------------------
' TextBox, Text Editor
'-------------------------------------------------------------------------------

Function TextBox_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Text As String, ByVal Style As ULong = 0, ByVal hwnd As HWND) As HWND
	
	'Create a new TextBox (single line). Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- text = Text
	'- Style: See below
	'- hwnd = handle of the window

	'Style:
	'- WS_TABSTOP
	'- ES_LEFT, ES_CENTER or ES_RIGHT
	'- ES_LOWERCASE or ES_UPPERCASE
	'- ES_NUMBER
	'- ES_PASSWORD
	'- ES_READONLY
	'(different styles can be connected by "Or")	
	
	Return CreateWindowEx(0, "EDIT", Text, Style Or WS_BORDER Or WS_VISIBLE Or WS_CHILD OR ES_AUTOHSCROLL, x, y, w, h, hWnd, 0, 0, 0 )
    	
End Function

	
Function TextEditor_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Text As String, ByVal Style As ULong = 0, ByVal hwnd As HWND) As HWND
	'Create a new Text Editor (multiline TextBox). Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Text = Text
	'- Style: ES_READONLY
	'- hwnd = handle of the window
	
	Return CreateWindowEx(0, "EDIT", Text, _
		Style Or WS_BORDER Or WS_VISIBLE Or WS_CHILD Or WS_HSCROLL Or WS_VSCROLL Or ES_MULTILINE, _
		x, y, w, h, hwnd, 0, 0, 0 )	
	                        
End Function


Sub TextBox_SetText(ByVal hWndText As HWND, ByVal Text As String)
	'Set text into an TextBox or an TextEditor. Parameters:
	'- hWndText = handle of the TextBox
	'- Text = text	
    
  SetWindowText(hWndText, Text)

End Sub


Function TextBox_GetText(ByVal hWndText As HWND) As String
	'Returns the text from an TextBox or an TextEditor. Parameter:
	'- hWndText = handle of the textbox
    
	Dim BufferSize As Long
	Dim Buffer As String
	
	BufferSize = GetWindowTextLength(hWndText)
	Buffer = Space(BufferSize)
	GetWindowText(hWndText, Buffer, BufferSize + 1)  
		 
	Return RTrim(Buffer, Chr(0))
         
End Function


Sub TextBox_Clear(ByVal hWndText As HWND)
	'Delets the current selection in an TextBox or an TextEditor. It does not
	'place the deleted text in the clipboard. Parameter:
	'- hWndText = handle of the TextBox
    
	SendMessage(hWndText, WM_CLEAR, 0, 0)
    
End Sub


Sub TextBox_Cut(ByVal hWndText As HWND)
	'Cuts the current selection in an TextBox or TextEditor  and places it in the
	'clipboard. Parameter:
	'- hWndText = Handle of the TextBox
  SendMessage(hWndText, WM_CUT, 0, 0)
    
End Sub


Sub TextBox_Copy(ByVal hWndText As HWND)
	'Copy the current selection in an TextBox or TextEditor and places it in the
	'clipboard. Parameters:
	'- hWndText = Handle of the TextBox
	
	SendMessage(hWndText, WM_COPY, 0, 0)
    
End Sub


Sub TextBox_Paste(ByVal hWndText As HWND)
	'Pastes the current clipboard contents. Parameter:
	'- hWndText = Handle of the TextBox

	SendMessage(hWndText, WM_PASTE, 0, 0)
    
End Sub


Sub TextBox_Undo(ByVal hWndText As HWND)
	'Undo the last operation. When this message is sent to an TextBox or an TextEditor,
	'the previously deleted text is restored or the previously added text is deleted.
	'Parameter:
	'- hWndText = handle of the TextBox
	
	SendMessage(hWndText, WM_UNDO, 0, 0)
	
End Sub 		


Function TextBox_GetLineCount(ByVal hWndText As HWND) As Long
'Returns the number of lines in an TextEditor. If no text exists then the return value is 1.
'Parameters:
'- hWndText: Handle of the TextEditor

	Return SendMessage(hWndText, EM_GETLINECOUNT, 0, 0)
    
End Function


Function TextBox_GetLine(ByVal hWndText As HWND, ByVal LineNumber As Integer) As String
'Gets a line of text from an TextEditor. Parameters: 
'- hWndText: Handle of TextEditor
'- LineNumber: The zero-based index of the line to retrieve.
    
	Dim BufferSize As Long
	Dim Buffer As String
	
	'Get the length of the text:
	BufferSize = SendMessage(hWndText, EM_LINELENGTH, LineNumber, 0)
	If BufferSize = 0 Then Return ""
	
	'Load the text into the buffer:
	Buffer = Space(BufferSize + 1)
	SendMessage hWndText, EM_GETLINE, LineNumber, Cast(LPARAM, Strptr(Buffer))
	
	Return RTrim(Buffer, Chr(0))
         
End Function


'-------------------------------------------------------------------------------
' ListBox
'-------------------------------------------------------------------------------

Function ListBox_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Style As ULong = 0, ByVal hwnd As HWND) As HWND
	'Create a new listbox. Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Style: e.g. LBS_USETABSTOPS for a multicolumn listbox 
	'- hwnd = handle of the window	
	
	Return CreateWindowEx(0, "LISTBOX", "", Style Or WS_BORDER Or WS_VISIBLE Or WS_CHILD Or WS_VSCROLL Or LBS_NOTIFY, _
		x, y, w, h, hWnd, 0, 0, 0 )
																							
End Function


Sub ListBox_AddString(ByVal hWndList As HWND, ByVal Text As String)
	'Add a line of text. Parameters:
	'- hWndList = Handle of the listbox
	'- Text = text to add	
	
	If text = "" Then Exit Sub
  SendMessage(hWndList, LB_ADDSTRING, 0, Cast(LPARAM, StrPtr(Text)))
    
End Sub


Sub Listbox_DeleteString(ByVal hWndList As HWND, ByVal Index As Long)
	'Delete string. Parameters:
	'- hWndList = handle of the listbox
	'- Index = index (starting from 0)	
    
  SendMessage(hWndList, LB_DELETESTRING, Index, 0)
    
End Sub


Sub  ListBox_InsertString(ByVal hWndList As HWND, ByVal Index As Long, ByVal Text As String)
	'Insert string. Parameters:
	'- hWndList = handle of the listbox
	'- Index = index (starting from 0). Index = -1: String is added at the end of 
	'  the list (equivalent to ListBox_AddString)
	'- Text = Text to be inserted
	
	If text = "" Then Exit Sub
	SendMessage(hWndList, LB_INSERTSTRING, Index, Cast(LPARAM, StrPtr(Text)))
    
End Sub


Sub ListBox_ReplaceString(ByVal hWndList As HWND, ByVal Index As Long, ByVal Text As String, _
                          ByVal NewData As Long = 0)	
	'Replace a string by another. Parameters:
	'- hWndList = Handle of the listbox
	'- Index = Index of the string to be replaced
	'- Text = new Text
	'- New Data = 32-bit value associated with the list box item 
	 
	 If text = "" Then Exit Sub 
   SendMessage(hWndList, LB_DELETESTRING, Index, 0)
   Index = SendMessage(hWndList, LB_INSERTSTRING, Index, Cast(LPARAM, Strptr(Text)))
   SendMessage(hWndList, LB_SETITEMDATA, Index, NewData)
        
End Sub                


Function ListBox_GetCount(ByVal hWndList As HWND ) As Long
	'Returns the number of entries in the listbox. The returned count is one greater 
	'than the index value of the last item (the index is zero-based).
	'Parameters:
	'- hWndList = Handle of the listbox	
    
  Return SendMessage(hWndList, LB_GETCOUNT, 0, 0)
    
End Function


Sub ListBox_ResetContent(ByVal hWndList As HWND)
	'Delete the contents of the listbox. Parameters:
	'- hWndList = Handle is the listbox	
    
  SendMessage(hWndList, LB_RESETCONTENT, 0, 0)
   
End Sub


Sub ListBox_SetCurSel(ByVal hWndList As HWND, ByVal Index As Long)
	'Set selection in the listbox. Parameters:
	'-hWndList = Handle of the listbox
	'- Index = index (starting from 0)
  	
  SendMessage(hWndList, LB_SETCURSEL, Index, 0)

End Sub            


Function ListBox_GetCurSel(ByVal hWndList As HWND ) As Long
	'Returns the selected index (index starts from 0). Parameters:
	'-hWndList = Handle of the listbox
  
  Return SendMessage(hWndList, LB_GETCURSEL, 0, 0)
    
End Function


Function ListBox_GetText(ByVal hWndList As HWND, ByVal Index As Long) As String
	'Returns the text corresponding to the index. Parameters:
	'-hWndList = handle of the listbox
	' - Index = hndex  	

	Dim BufferSize As Long
	Dim Buffer As String
	
	'Examine text length:
	BufferSize = SendMessage(hWndList, LB_GETTEXTLEN, Index, 0)
	Buffer = Space(BufferSize + 1)

	'Load text into the buffer:
	If SendMessage(hWndList, LB_GETTEXT, Index, Cast(LPARAM, StrPtr(Buffer))) = LB_ERR Then
		Return ""
	Else
		Return RTrim(Buffer, Chr(0))
	End If      
			
End Function


Sub ListBox_SetTabstops(ByVal hWndList As HWND, TabStops() As Integer) 
	'Set Tabstops into a listbox. Parameters:
	'- hWndList = handle of the listbox
	'- array of integers containing the tab stops.
	'The listbox must have been created with the LBS_USETABSTOPS style.
	 	 
	SendMessage(hWndList, LB_SETTABSTOPS, 1, Cast(LParam, @(TabStops(0))))
	
End Sub


'-------------------------------------------------------------------------------
' ComboBox
'-------------------------------------------------------------------------------

Function ComboBox_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Style As Ulong = 0, ByVal hwnd As HWND) As HWND
	'New Combobox. Parameters:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Style = style
	'- hwnd = handle of the window
	
	Return CreateWindow("COMBOBOX", "", WS_BORDER Or WS_VISIBLE Or WS_CHILD Or WS_VSCROLL Or CBS_DROPDOWNLIST, _
		x, y, w, h, hWnd, 0, 0, 0 )	
																						
End Function


Sub ComboBox_AddString(ByVal hWndCombo As HWND, ByVal Text As String)
	'Add a line of text. Parameters:
	'- hWndCombo = handle of combo box
	'- Text = text
  
  If text = "" Then Exit Sub  
  SendMessage(hWndCombo, CB_ADDSTRING, 0, Cast(LPARAM, StrPtr(Text)) )
     
End Sub


Sub ComboBox_DeleteString(ByVal hWndCombo As HWND, ByVal Index As Long)
	'Delete string. Parameters:
	'- hWndCombo = handle of the ComboBox
	'- Index = index (starting from 0)   
  
	SendMessage(hWndCombo, CB_DELETESTRING, Index, 0)
   
End Sub

'-------------------------------------------------------------------------------

Sub  ComboBox_InsertString(ByVal hWndCombo As HWND, ByVal Index As Long, ByVal Text As String)
   'Insert string. Parameters:
   '- hWndCombo = handle of the ComboBox
   '- Index = index (starting from 0). Index = -1: String is added at the end of
   '  the list (equivalent to ComboBox_AddString)
   '- Text = Text to be inserted
   
   If text = "" Then Exit Sub
   SendMessage(hWndCombo, CB_INSERTSTRING, Index, Cast(LPARAM, Strptr(Text)))
   
End Sub

'-------------------------------------------------------------------------------

Sub ComboBox_ReplaceString(ByVal hWndCombo As HWND, ByVal Index As Long, ByVal Text As String, _
   ByVal NewData As Long = 0)   
   'Replace a string by another. Parameters:
   '- hWndCombo = Handle of the ComboBox
   '- Index = Index of the string to be replaced
   '- Text = new Text
   '- New Data = 32-bit value associated with the combo box item
   
   If text = "" Then Exit Sub  
   SendMessage(hWndCombo, CB_DELETESTRING, Index, 0)
   Index = SendMessage(hWndCombo, CB_INSERTSTRING, Index, Cast(LPARAM, Strptr(Text)))
   SendMessage(hWndCombo, CB_SETITEMDATA, Index, NewData)
   
End Sub

'-------------------------------------------------------------------------------

Function ComboBox_GetCount(ByVal hWndCombo As HWND ) As Long
   'Returns the number of entries in the ComboBox. The returned count is one greater
   'than the index value of the last item (the index is zero-based).
   'Parameters:
   '- hWndCombo = Handle of the ComboBox   
   
   Return SendMessage(hWndCombo, CB_GETCOUNT, 0, 0)
   
End Function

'-----------------------------------------------------------------------------

Sub ComboBox_ResetContent(ByVal hWndCombo As HWND)
   'Delete the contents of the ComboBox. Parameters:
   '- hWndCombo = Handle is the ComboBox   
   
   SendMessage(hWndCombo, CB_RESETCONTENT, 0, 0)
   
End Sub


Sub ComboBox_SetCurSel(ByVal hWndCombo As HWND, ByVal Index As Long)
	'Set the current selection of the combo box. Parameters:
  '- hWndCombo = handle of combo box 
  '- Index = index (beginning from 0)
  
  SendMessage(hWndCombo, CB_SETCURSEL, Index, 0)
  	
End Sub
    

Function ComboBox_GetCurSel(ByVal hWndCombo As HWND ) As Long
  'Returns the selected index (beginning from 0. Parameters:
  '-hWndCombo = handle of combo box
  	
  Return SendMessage(hWndCombo, CB_GETCURSEL, 0, 0)
     
End Function


Function ComboBox_GetText(ByVal hWndCombo As HWND, ByVal Index As Long) As String
	'Returns the text corresponding to the index. Parameters:
	'- hWndCombo = handle of the combo box
	'- Index = hndex
  
  Dim BufferSize As Long
  Dim Buffer As String
    
	'examine text length:
	BufferSize = SendMessage(hWndCombo, CB_GETLBTEXTLEN, Index, 0)
 	Buffer = Space(BufferSize + 1)
 	
 	'load text into the buffer:
 	If SendMessage(hWndCombo, CB_GETLBTEXT, Index, Cast(LPARAM, StrPtr(Buffer))) = CB_ERR Then
 		Return ""
 	Else
 		Return RTrim(Buffer, Chr(0))
 	End If 
 	                      
End Function            


'-------------------------------------------------------------------------------
' Trackbar
'-------------------------------------------------------------------------------

Function TrackBar_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Style As ULong = TBS_HORZ, ByVal hwnd As HWND) As HWND
	'Create a new trackbar. Parameter:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Style: See below
	'- hwnd = handle of the window
	
	'Styles:
	'TBS_HORZ or TBS_VERT: horizontal or vertical trackbar
   	
	Return CreateWindowEx( 0, TRACKBAR_CLASS, "", Style Or WS_VISIBLE Or WS_CHILD Or TBS_ENABLESELRANGE, _
		x, y, w, h, hWnd, 0, 0, 0)
				
End Function


Sub TrackBar_SetRange(ByVal hWndTrack As HWND, ByVal RangeMin As Long, ByVal RangeMax As Long)
	'Set range of the track bar. Parameters:
	'- hWnd = handle of the track bar
	'- RangeMin, RangeMax = Range
	
	SendMessage(hWndTrack, TBM_SETRANGEMIN, 0, RangeMin)
	SendMessage(hWndTrack, TBM_SETRANGEMAX, 0, RangeMax)
	
End Sub


Function TrackBar_GetPos(ByVal hWndTrack As HWND) As Long
	'Returns the position of the track bar. Parameter:
	'- hWndTrack = handle of the track bar

	Return SendMessage(hWndTrack, TBM_GETPOS, 0, 0 )
	
End Function


Sub TrackBar_SetPos(ByVal hWndTrack As HWND, ByVal NewPos As Long)
   'Set the position of the trackbar. Parameters:
   '- hWndTrack = handle of the track bar
   '- NewPos = new position

	SendMessage(hWndTrack, TBM_SETPOS, TRUE, NewPos)
	
End Sub


'-------------------------------------------------------------------------------
' Progressbar
'-------------------------------------------------------------------------------
		

Function ProgressBar_New(ByVal x As Long, ByVal y As Long, ByVal w As Long, ByVal h As Long, _
	ByVal Style As Ulong = 0, ByVal hWnd As HWND) As HWND
	'Create new Progressbar. Parameter:
	'- x, y = top left corner
	'- w = width
	'- h = height
	'- Style = style
	'- hwnd = handle of the window	
	
	Return CreateWindowEx(0, PROGRESS_CLASS, "", Style Or WS_VISIBLE Or WS_CHILD, x, y, w, h, hWnd, 0, 0, 0)
												
End Function


Sub ProgressBar_SetRange(ByVal hWndProg As HWND, ByVal RangeMin As Long, ByVal RangeMax As Long)
	'Set range of a progress bars. Parameters:
	'- hWndProgessbar = handle of the progress bar
	'- RangeMin, RangeMax = Range
	
	SendMessage(hWndProg, PBM_SETRANGE, 0, MAKELPARAM(RangeMin, RangeMax))
	
End Sub


Sub ProgressBar_SetPos(ByVal hWndProg As HWND, ByVal NewPos As Long)
  'Set the position of a progress bar. Parameter:
   '- hWndProgessbar = handle of the progress bar
   '- NewPos = new position
	
	SendMessage(hWndProg, PBM_SETPOS, NewPos, 0)
	
End Sub


'-------------------------------------------------------------------------------
' Menu
'-------------------------------------------------------------------------------

'Method how to create a menu:
'- Create handle of the menu and all handles of the menu items by CreateMenu()
'- Create all menu titles (Sub MenuTitle(), see below)
'- Create all menu items (Sub MenuItem(), see below)
'- Assign menu to the window by SetMenu() 
 
Sub MenuTitle(ByVal hMenu As HMENU, ByVal hMenuTitle As HMENU, ByVal Title As String)
   'Menu heading. Parameters:
   '- hMenu = handle of the menu
   '- hMenuTitle = handle of the relevant menu item (for example, hFile)
   '- Title = title of the menu item (for example, "File")
  
  InsertMenu(hMenu, 0, MF_POPUP, CInt(hMenuTitle), Title)
  
End Sub


Sub MenuItem(ByVal hMenuTitle As HMENU, ByVal MenuItemNumber As Long, ByVal MenuItemName As String)
   'Menu item (entry into a menu). Parameters:
   '- hMenuTitle = handle of the parent menu item (see Sub MenuTitle, e.g. hFile)
   '- MenuItemNumber = number of the entry
   '- MenuItemName = name of the entry (eg "open")
  
  AppendMenu(hMenuTitle, 0, MenuItemNumber, MenuItemName)
  
End Sub


